//+------------------------------------------------------------------+
//|                                          EGEN Random Advisor.mq4 |
//|                             Copyright  2010, EGEN Software LTD. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+

#property copyright "Copyright  2010, EGEN Software LTD."
#property link      "http://www.metaquotes.net"

extern   bool     CloseAll    =  false;
extern   double   TakeProfit  =  0;
extern   double   StopLoss    =  0;
extern   double   Lots        =  0;
extern   int      Slippage    =  3;

bool EQ(int value1, int value2)
{
   return(value1==value2);
}

int R(int value)
{
   return(MathRand()%value);
}

double INFO(int type)
{
   return(MarketInfo(Symbol(),type));
}

double MINLOT()
{
   return(INFO(MODE_MINLOT));
}

double LOTS()
{   
   if(Lots>0) return(Lots); else return(MINLOT());
}

datetime MAGIC()
{
   return(R(PERIOD_D1)*60);
}

void Buy()
{ 
   OrderSend(Symbol(),OP_BUY,LOTS(),Ask,Slippage,0,0,NULL,MAGIC()); 
}

void Sell()
{
   OrderSend(Symbol(),OP_SELL,LOTS(),Bid,Slippage,0,0,NULL,MAGIC());   
}

void BUY()
{ 
   OrderSend(Symbol(),OP_BUY,OrderLots()*2,Ask,Slippage,0,0,NULL,OrderMagicNumber()*2); 
}

void SELL()
{
   OrderSend(Symbol(),OP_SELL,OrderLots()*2,Bid,Slippage,0,0,NULL,OrderMagicNumber()*2);   
}

datetime EXPIRATION()
{
   return(OrderOpenTime()+OrderMagicNumber());
}

bool CheckExpiration()
{
   for(int i=0; TOTAL(i); i++)
      if(SelectOrder(i))
         if(EXPIRATION()>TimeLocal()) return(false);
      else continue;
   return(true);               
}

double PROFIT()
{
   return(OrderProfit()/OrderLots());
}

int LOSS(int ticket=EMPTY)
{
   static double PROFIT;
   if(ticket<0) PROFIT=0; else if(PROFIT()<PROFIT) PROFIT=PROFIT();
   return(ticket);          
}

bool SelectLoss()
{
   int ticket=LOSS();   
   for(int i=0; TOTAL(i); i++)
      if(SelectOrder(i)) ticket=LOSS(OrderTicket());
   if(SelectTicket(ticket)) return(true);               
   return(false);         
}

bool CheckSymbol()
{
   for(int i=0; TOTAL(i); i++)
      if(SelectOrder(i)) return(false);         
   return(true);      
}

void COMMENT()
{
   Comment("\n"+TimeToStr(EXPIRATION()));      
}

void CheckProfit()
{                 
   if(CheckSymbol())   
      SendOrder(); else
   if(CheckExpiration()) 
      if(AccountProfit()>0) CloseAll(); 
         else        
      if(SelectLoss()) LockOrder(); else return;
   else COMMENT();
}

int TREND()
{
   if(R(2)>0) return(1); else return(-1);   
}
   
void SendOrder() 
{ 
   if(TREND()<0) Sell(); else Buy();    
}              

void LockOrder()
{
   if(ORDER()<0) BUY(); else SELL(); 
}

bool TOTAL(int index)
{
   return(index<OrdersTotal());
}

bool SYMBOL()
{
   return(OrderSymbol()==Symbol());
}

bool SelectOrder(int index) 
{
   if(OrderSelect(index,SELECT_BY_POS)) 
      if(SYMBOL()) return(true);
   return(false);         
}

bool SelectTicket(int ticket) 
{
   if(OrderSelect(ticket,SELECT_BY_TICKET)) 
      if(SYMBOL()) return(true);
   return(false);         
}

int ORDER()
{   
   if(EQ(OrderType(),OP_SELL))       
      return(-1); 
   if(EQ(OrderType(),OP_BUY)) 
      return(1); 
   return(0);      
}  

double PRICE()
{
   if(ORDER()>0) return(Bid);
      else 
   if(ORDER()<0) return(Ask);     
}

void CloseTicket()
{
   OrderClose(OrderTicket(),OrderLots(),PRICE(),Slippage);              
}

void CloseOrder(int index)
{
   if(SelectOrder(index)) CloseTicket();
}

void CloseAll()
{
   for(int i=0; TOTAL(i); i++) CloseOrder(i); 
}

bool TakeProfit()
{
   if(TakeProfit>0)
      return(AccountEquity()>TakeProfit); 
   return(false);      
}

bool StopLoss()
{
   if(StopLoss>0)
      return(AccountEquity()<StopLoss);
   return(false);      
}

void CheckTrade()
{
   if(CloseAll) 
      CloseAll(); else
   if(TakeProfit()) 
      CloseAll=true; else
   if(StopLoss()) 
      CloseAll=true; 
   else CheckProfit(); 
}

void init()
{
   MathSrand(TimeLocal());
}

void start()
{
   if(IsTradeAllowed()) CheckTrade();
}



